@@ -2,7 +2,7 @@
  * vlc.c: the VLC player
  *****************************************************************************
  * Copyright (C) 1998-2008 the VideoLAN team
- * $Id: b99b5aab5762d81855473772d90a2f65c985d03d $
+ * $Id: 51633b2bff13685e4ce1a02a4636988c3541f87a $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
@@ -68,6 +68,8 @@ int main( int i_argc, const char *ppsz_argv[] )
      * Note: this is NOT an excuse for not protecting against SIGPIPE. If
      * LibVLC runs outside of VLC, we cannot rely on this code snippet. */
     signal (SIGPIPE, SIG_IGN);
+    /* Restore default for SIGCHLD in case parent ignores it. */
+    signal (SIGCHLD, SIG_DFL);
 
 #ifdef HAVE_SETENV
 # ifndef NDEBUG
@@ -78,10 +80,6 @@ int main( int i_argc, const char *ppsz_argv[] )
     setenv ("GNOME_DISABLE_CRASH_DIALOG", "1", 1);
 # endif
 
-    /* Make Xlib hide visuals with an alphachannel. Ensure that Qt4 will not
-     * use the alpha channel for the embedded video window. */
-    setenv ("XLIB_SKIP_ARGB_VISUALS", "1", 1);
-
     /* Clear the X.Org startup notification ID. Otherwise the UI might try to
      * change the environment while the process is multi-threaded. That could
      * crash. Screw you X.Org. Next time write a thread-safe specification. */
@@ -184,9 +182,11 @@ int main( int i_argc, const char *ppsz_argv[] )
 
     if (vlc != NULL)
     {
+        libvlc_set_user_agent (vlc, "VLC media player", NULL);
+
         if (libvlc_add_intf (vlc, "signals"))
             pthread_sigmask (SIG_UNBLOCK, &set, NULL);
-#if !defined (HAVE_MAEMO)
+#if !defined (HAVE_MAEMO) && !defined __APPLE__
         libvlc_add_intf (vlc, "globalhotkeys,none");
 #endif
         if (libvlc_add_intf (vlc, NULL) == 0)
@@ -200,14 +200,5 @@ int main( int i_argc, const char *ppsz_argv[] )
     for (int i = 1; i < argc; i++)
         LocaleFree (argv[i]);
 
-#ifdef RTLD_NOLOAD
-    /* Avoid crash in KIO scheduler cleanup. */
-    /* This is ugly, but we get way too many crash reports due to this. */
-    if (dlopen ("libkfilemodule.so", RTLD_LAZY|RTLD_LOCAL|RTLD_NOLOAD) != NULL)
-    {
-        fprintf (stderr, "KFile plugin present. Unclean shutdown!\n");
-        _exit (0);
-    }
-#endif
     return 0;
 }
