@@ -1,8 +1,8 @@
 /*****************************************************************************
  * b4s.c : B4S playlist format import
  *****************************************************************************
- * Copyright (C) 2005 the VideoLAN team
- * $Id: be33b7826fb3b151669bb09d1fed8b6cdae19dc8 $
+ * Copyright (C) 2005-2009 the VideoLAN team
+ * $Id: 6b889f402d5fd39871c6febc54ba70e5632aaf3d $
  *
  * Authors: Sigmund Augdal Helberg <dnumgis@videolan.org>
  *
@@ -31,16 +31,13 @@
 
 #include <vlc_common.h>
 #include <vlc_demux.h>
-#include <vlc_interface.h>
+#include <vlc_xml.h>
 
 #include "playlist.h"
-#include "vlc_xml.h"
 
 struct demux_sys_t
 {
     char *psz_prefix;
-    xml_t *p_xml;
-    xml_reader_t *p_xml_reader;
 };
 
 /*****************************************************************************
@@ -48,7 +45,6 @@ struct demux_sys_t
  *****************************************************************************/
 static int Demux( demux_t *p_demux);
 static int Control( demux_t *p_demux, int i_query, va_list args );
-//static char *GetNextToken(char *psz_cur_string);
 static int IsWhitespace( char *psz_string );
 
 /*****************************************************************************
@@ -59,8 +55,6 @@ int Import_B4S( vlc_object_t *p_this )
     DEMUX_BY_EXTENSION_OR_FORCED_MSG( ".b4s", "b4s-open",
                                       "using B4S playlist reader" );
     p_demux->p_sys->psz_prefix = FindPrefix( p_demux );
-    p_demux->p_sys->p_xml = NULL;
-    p_demux->p_sys->p_xml_reader = NULL;
     return VLC_SUCCESS;
 }
 
@@ -73,43 +67,41 @@ void Close_B4S( vlc_object_t *p_this )
     demux_sys_t *p_sys = p_demux->p_sys;
 
     free( p_sys->psz_prefix );
-    if( p_sys->p_xml_reader ) xml_ReaderDelete( p_sys->p_xml, p_sys->p_xml_reader );
-    if( p_sys->p_xml ) xml_Delete( p_sys->p_xml );
     free( p_sys );
 }
 
 static int Demux( demux_t *p_demux )
 {
-    demux_sys_t *p_sys = p_demux->p_sys;
-    int i_ret;
+    int i_ret = -1;
 
     xml_t *p_xml;
-    xml_reader_t *p_xml_reader;
+    xml_reader_t *p_xml_reader = NULL;
     char *psz_elname = NULL;
-    int i_type;
     input_item_t *p_input;
-    char *psz_mrl = NULL, *psz_name = NULL, *psz_genre = NULL;
+    char *psz_mrl = NULL, *psz_title = NULL, *psz_genre = NULL;
     char *psz_now = NULL, *psz_listeners = NULL, *psz_bitrate = NULL;
+    input_item_node_t *p_subitems = NULL;
 
-    INIT_PLAYLIST_STUFF;
+    input_item_t *p_current_input = GetCurrentItem(p_demux);
 
-    p_xml = p_sys->p_xml = xml_Create( p_demux );
-    if( !p_xml ) return -1;
+    p_xml = xml_Create( p_demux );
+    if( !p_xml )
+        goto end;
 
     psz_elname = stream_ReadLine( p_demux->s );
     free( psz_elname );
     psz_elname = NULL;
 
     p_xml_reader = xml_ReaderCreate( p_xml, p_demux->s );
-    if( !p_xml_reader ) return -1;
-    p_sys->p_xml_reader = p_xml_reader;
+    if( !p_xml_reader )
+        goto end;
 
     /* xml */
     /* check root node */
     if( xml_ReaderRead( p_xml_reader ) != 1 )
     {
         msg_Err( p_demux, "invalid file (no root node)" );
-        return -1;
+        goto end;
     }
 
     if( xml_ReaderNodeType( p_xml_reader ) != XML_READER_STARTELEM ||
@@ -118,10 +110,9 @@ static int Demux( demux_t *p_demux )
     {
         msg_Err( p_demux, "invalid root node %i, %s",
                  xml_ReaderNodeType( p_xml_reader ), psz_elname );
-        free( psz_elname );
-        return -1;
+        goto end;
     }
-    free( psz_elname );
+    FREENULL( psz_elname );
 
     /* root node should not have any attributes, and should only
      * contain the "playlist node */
@@ -132,17 +123,16 @@ static int Demux( demux_t *p_demux )
     if( i_ret != 1 )
     {
         msg_Err( p_demux, "invalid file (no child node)" );
-        return -1;
+        goto end;
     }
 
     if( ( psz_elname = xml_ReaderName( p_xml_reader ) ) == NULL ||
         strcmp( psz_elname, "playlist" ) )
     {
         msg_Err( p_demux, "invalid child node %s", psz_elname );
-        free( psz_elname );
-        return -1;
+        goto end;
     }
-    free( psz_elname ); psz_elname = NULL;
+    FREENULL( psz_elname );
 
     // Read the attributes
     while( xml_ReaderNextAttr( p_xml_reader ) == VLC_SUCCESS )
@@ -153,7 +143,7 @@ static int Demux( demux_t *p_demux )
         {
             free( psz_name );
             free( psz_value );
-            return -1;
+            goto end;
         }
         if( !strcmp( psz_name, "num_entries" ) )
         {
@@ -172,43 +162,48 @@ static int Demux( demux_t *p_demux )
         free( psz_value );
     }
 
+    p_subitems = input_item_node_Create( p_current_input );
+
     while( (i_ret = xml_ReaderRead( p_xml_reader )) == 1 )
     {
         // Get the node type
-        i_type = xml_ReaderNodeType( p_xml_reader );
-        switch( i_type )
+        switch( xml_ReaderNodeType( p_xml_reader ) )
         {
             // Error
             case -1:
-                return -1;
-                break;
+                goto end;
 
             case XML_READER_STARTELEM:
             {
                 // Read the element name
                 free( psz_elname );
                 psz_elname = xml_ReaderName( p_xml_reader );
-                if( !psz_elname ) return -1;
-
+                if( !psz_elname )
+                    goto end;
 
                 // Read the attributes
                 while( xml_ReaderNextAttr( p_xml_reader ) == VLC_SUCCESS )
                 {
                     char *psz_name = xml_ReaderName( p_xml_reader );
                     char *psz_value = xml_ReaderValue( p_xml_reader );
-                    if( !psz_name || !psz_value ) return -1;
+                    if( !psz_name || !psz_value )
+                    {
+                        free( psz_name );
+                        free( psz_value );
+                        goto end;
+                    }
                     if( !strcmp( psz_elname, "entry" ) &&
                         !strcmp( psz_name, "Playstring" ) )
                     {
-                        psz_mrl = strdup( psz_value );
+                        psz_mrl = psz_value;
                     }
                     else
                     {
-                        msg_Warn( p_demux, "unexpected attribure %s in element %s",
+                        msg_Warn( p_demux, "unexpected attribute %s in element %s",
                                   psz_name, psz_elname );
+                        free( psz_value );
                     }
                     free( psz_name );
-                    free( psz_value );
                 }
                 break;
             }
@@ -222,34 +217,34 @@ static int Demux( demux_t *p_demux )
                 }
                 if( !strcmp( psz_elname, "Name" ) )
                 {
-                    psz_name = strdup( psz_text );
+                    psz_title = psz_text;
                 }
                 else if( !strcmp( psz_elname, "Genre" ) )
                 {
-                    psz_genre = strdup( psz_text );
+                    psz_genre = psz_text;
                 }
                 else if( !strcmp( psz_elname, "Nowplaying" ) )
                 {
-                    psz_now = strdup( psz_text );
+                    psz_now = psz_text;
                 }
                 else if( !strcmp( psz_elname, "Listeners" ) )
                 {
-                    psz_listeners = strdup( psz_text );
+                    psz_listeners = psz_text;
                 }
                 else if( !strcmp( psz_elname, "Bitrate" ) )
                 {
-                    psz_bitrate = strdup( psz_text );
+                    psz_bitrate = psz_text;
                 }
                 else if( !strcmp( psz_elname, "" ) )
                 {
-                    ;
+                    free( psz_text );
                 }
                 else
                 {
                     msg_Warn( p_demux, "unexpected text in element '%s'",
                               psz_elname );
+                    free( psz_text );
                 }
-                free( psz_text );
                 break;
             }
             // End element
@@ -258,10 +253,11 @@ static int Demux( demux_t *p_demux )
                 // Read the element name
                 free( psz_elname );
                 psz_elname = xml_ReaderName( p_xml_reader );
-                if( !psz_elname ) return -1;
+                if( !psz_elname )
+                    goto end;
                 if( !strcmp( psz_elname, "entry" ) )
                 {
-                    p_input = input_item_New( p_demux, psz_mrl, psz_name );
+                    p_input = input_item_New( p_demux, psz_mrl, psz_title );
                     if( psz_now )
                         input_item_SetNowPlaying( p_input, psz_now );
                     if( psz_genre )
@@ -271,9 +267,9 @@ static int Demux( demux_t *p_demux )
                     if( psz_bitrate )
                         msg_Err( p_demux, "Unsupported meta bitrate" );
 
-                    input_item_AddSubItem( p_current_input, p_input );
+                    input_item_node_AppendItem( p_subitems, p_input );
                     vlc_gc_decref( p_input );
-                    FREENULL( psz_name );
+                    FREENULL( psz_title );
                     FREENULL( psz_mrl );
                     FREENULL( psz_genre );
                     FREENULL( psz_bitrate );
@@ -281,7 +277,7 @@ static int Demux( demux_t *p_demux )
                     FREENULL( psz_now );
                 }
                 free( psz_elname );
-                psz_elname = strdup("");
+                psz_elname = strdup( "" );
 
                 break;
             }
@@ -291,12 +287,21 @@ static int Demux( demux_t *p_demux )
     if( i_ret != 0 )
     {
         msg_Warn( p_demux, "error while parsing data" );
+        i_ret = 0; /* Needed for correct operation of go back */
     }
 
-   free( psz_elname );
+end:
+    free( psz_elname );
 
-    HANDLE_PLAY_AND_RELEASE;
-    return 0; /* Needed for correct operation of go back */
+    if( p_subitems )
+        input_item_node_PostAndDelete( p_subitems );
+
+    vlc_gc_decref( p_current_input );
+    if( p_xml_reader )
+        xml_ReaderDelete( p_xml, p_xml_reader );
+    if( p_xml )
+        xml_Delete( p_xml );
+    return i_ret;
 }
 
 static int Control( demux_t *p_demux, int i_query, va_list args )
@@ -305,29 +310,6 @@ static int Control( demux_t *p_demux, int i_query, va_list args )
     return VLC_EGENERIC;
 }
 
-#if 0
-/**
- * Get a in-string pointer to the start of the next token from a
- * string terminating the pointer returned by a previous call.
- *
- * \param psz_cur_string The string to search for the token from
- * \return a pointer to withing psz_cur_string, or NULL if no token
- * was found
- * \note The returned pointer may contain more than one
- * token, Run GetNextToken once more to terminate the token properly
- */
-static char *GetNextToken(char *psz_cur_string) {
-    while (*psz_cur_string && !isspace(*psz_cur_string))
-        psz_cur_string++;
-    if (!*psz_cur_string)
-        return NULL;
-    *psz_cur_string++ = '\0';
-    while (*psz_cur_string && isspace(*psz_cur_string))
-        psz_cur_string++;
-    return psz_cur_string;
-}
-#endif
-
 static int IsWhitespace( char *psz_string )
 {
     while( *psz_string )
